# load required library
library(rpart)



########################################################################
# tree.learn
#
# This function grows a decision tree for the given training set using
# the rpart package, then evaluates the tree on both training and test
# data, returning the resulting predictions.
#
# (You may modify this function to return *additional* values in the
# returned list.  You also may modify the function to take additional
# *optional* input parameters, provided that the default values of
# these additional parameters cause the function to behave exactly
# like this one that we are providing.)
#
# Input parameters:
#
#    train.set  --  a data frame containing the training data
#    test.set   --  a data frame containing the test data
#    class.name --  (optional) the name of the target variable in the
#                   data frame that is to be modeled, i.e., regarded
#                   as the class or label (= "Class" by default)
#    weights    --  (optional) a vector specifying how much weight is
#                   to be assigned to each training example
#    maxdepth   --  (optional) the maximum depth of the decision trees
#                   being grown
#
# Returns a list with the following values:
#
#    train.pred --  the predictions of the computed decision tree on
#                   all of the training examples
#    test.pred  --  the predictions of the computed decision tree on
#                   all of the test examples
########################################################################
tree.learn <- function(train.set,
                       test.set,
                       class.name="Class",
                       weights=rep(1,times= dim(train.set)[1]),
                       maxdepth=30) {

  form <- as.formula(paste(class.name, " ~ ",
                           paste(names(train.set),collapse="+"),
                           "-", class.name))

  tree <- rpart(form, train.set,
                weights=weights,
                method="class",
                control=rpart.control(xval=1,maxdepth=maxdepth))

  train.pred <- predict(tree, train.set, type="class")
  test.pred  <- predict(tree, test.set,  type="class")

  list(train.pred=train.pred, test.pred=test.pred)
  
}



########################################################################
# boost.trees
#
# This is the template for the function boost.trees which you need to
# fill in.  This function should run boosting on the tree.learn
# procedure for T rounds.
#
# Input parameters:
#
#    train.set  --  a data frame containing the training data
#    test.set   --  a data frame containing the test data
#    T          --  number of rounds of boosting
#    class.name --  (optional) the name of the target variable in the
#                   data frame that is to be modeled, i.e., regarded
#                   as the class or label (= "Class" by default)
#    maxdepth   --  (optional) the maximum depth of the decision trees
#                   being grown
#
# Returns a list with the following values:
#
#    train.pred --  the predictions of the combined classifier (after
#                   all T rounds) on all of the training examples
#    test.pred  --  the predictions of the combined classifier (after
#                   all T rounds) on all of the test examples
#    train.err  --  a vector of length T whose t-th component gives
#                   the training error of the combined classifier
#                   after t rounds
#    test.err  --   a vector of length T whose t-th component gives
#                   the training error of the combined classifier
#                   after t rounds
########################################################################
boost.trees <- function(train.set, test.set, T,
                        class.name="Class", maxdepth=30) {

  #
  # fill in your code here
  #


  # return everything as a list
  list(
       train.pred=    , # fill in
       test.pred=     , # fill in
       train.err=     , # fill in
       test.err=        # fill in
       )
}


       
########################################################################
# bag.trees
#
# This is the template for the function bag.trees which you need to
# fill in.  This function should run bagging on the tree.learn
# procedure for T rounds.
#
# Input parameters:
#
#    train.set  --  a data frame containing the training data
#    test.set   --  a data frame containing the test data
#    T          --  number of rounds of bagging
#    class.name --  (optional) the name of the target variable in the
#                   data frame that is to be modeled, i.e., regarded
#                   as the class or label (= "Class" by default)
#    maxdepth   --  (optional) the maximum depth of the decision trees
#                   being grown
#
# Returns a list with the following values:
#
#    train.pred --  the predictions of the combined classifier (after
#                   all T rounds) on all of the training examples
#    test.pred  --  the predictions of the combined classifier (after
#                   all T rounds) on all of the test examples
#    train.err  --  a vector of length T whose t-th component gives
#                   the training error of the combined classifier
#                   after t rounds
#    test.err  --   a vector of length T whose t-th component gives
#                   the training error of the combined classifier
#                   after t rounds
########################################################################
bag.trees <- function(train.set, test.set, T,
                      class.name="Class", maxdepth=30) {

  #
  # fill in your code here
  #


  # return everything as a list
  list(
       train.pred=    , # fill in
       test.pred=     , # fill in
       train.err=     , # fill in
       test.err=        # fill in
       )
}
